﻿//   Copyright 2023 Esri
//   Licensed under the Apache License, Version 2.0 (the "License");
//   you may not use this file except in compliance with the License.
//   You may obtain a copy of the License at

//       https://www.apache.org/licenses/LICENSE-2.0

//   Unless required by applicable law or agreed to in writing, software
//   distributed under the License is distributed on an "AS IS" BASIS,
//   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
//   See the License for the specific language governing permissions and
//   limitations under the License. 

using ArcGIS.Desktop.Editing.Attributes;
using ArcGIS.Desktop.Framework;
using ArcGIS.Desktop.Framework.Controls;
using ArcGIS.Desktop.Mapping;
using System.Collections.Generic;
using System.Windows.Controls;
using System.Windows.Input;
using System.Xml.Linq;


namespace EditingCustomizations
{
  internal class AttributeControlViewModel : EmbeddableControl
  {
    private EmbeddableControl _inspectorViewModel = null;
    private UserControl _inspectorView = null;
    private Inspector _featureInspector = null;
    private MyProvider _provider = null;

    public AttributeControlViewModel(XElement options, bool canChangeOptions) : base(options, canChangeOptions) 
    {
      // the standard way of creating an inspector
      //_featureInspector = new Inspector();


      // **  an alternative way to create an inspector
      //   allowing us to customize the grid view

      // create the provider
      _provider = new MyProvider();
      // create the inspector from the provider
      _featureInspector = _provider.Create();

      _featureInspector.PropertyChanged += _featureInspector_PropertyChanged;

      // create the embeddable control from the inspector (to display on the pane)
      var icontrol = _featureInspector.CreateEmbeddableControl();

      // get view and viewmodel from the inspector
      InspectorViewModel = icontrol.Item1;
      InspectorView = icontrol.Item2;
    }

    private void _featureInspector_PropertyChanged(object sender, System.ComponentModel.PropertyChangedEventArgs e)
    {
      if (e.PropertyName == "IsDirty")
      {
        NotifyPropertyChanged(nameof(IsApplyEnabled));
        NotifyPropertyChanged(nameof(IsCancelEnabled));
      }
    }

    internal void Cleanup()
    {
      InspectorView = null;
      InspectorViewModel.Dispose();
    }

    /// <summary>
    /// Property containing an instance for the inspector.
    /// </summary>
    public Inspector AttributeInspector => _featureInspector;

    public EmbeddableControl InspectorViewModel
    {
      get => _inspectorViewModel; 
      set
      {
        if (value != null)
        {
          _inspectorViewModel = value;
          _inspectorViewModel.OpenAsync();

        }
        else if (_inspectorViewModel != null)
        {
          _inspectorViewModel.CloseAsync();
          _inspectorViewModel = value;
        }
        NotifyPropertyChanged(nameof(InspectorViewModel));
      }
    }

    /// <summary>
    /// Property for the inspector UI.
    /// </summary>
    public UserControl InspectorView
    {
      get => _inspectorView; 
      set => SetProperty(ref _inspectorView, value); 
    }

    private Dictionary<MapMember, List<long>> _selection = null;

    /// <summary>
    /// Dictionary holding the selected features in the map to populate the tree view for 
    /// layers and respective selected features.
    /// </summary>
    public Dictionary<MapMember, List<long>> SelectedMapFeatures
    {
      get => _selection;
      set => SetProperty(ref _selection, value);
    }

    public bool IsApplyEnabled => AttributeInspector?.IsDirty ?? false;

    public bool IsCancelEnabled => AttributeInspector?.IsDirty ?? false;

    private ICommand _cancelCommand;
    public ICommand CancelCommand
    {
      get
      {
        if (_cancelCommand == null)
          _cancelCommand = new RelayCommand(OnCancel);
        return _cancelCommand;
      }
    }

    internal void OnCancel()
    {
      AttributeInspector?.Cancel();
    }

    private ICommand _applyCommand;
    public ICommand ApplyCommand
    {
      get
      {
        if (_applyCommand == null)
          _applyCommand = new RelayCommand(OnApply);
        return _applyCommand;
      }
    }

    internal void OnApply()
    {
      AttributeInspector?.Apply();
    }
  }
}
